// kaldifst/python/csrc/lattice-utils.h
//
// Copyright (c)  2021-2023  Xiaomi Corporation (authors: Fangjun Kuang)

#ifndef KALDIFST_PYTHON_CSRC_LATTICE_UTILS_H_
#define KALDIFST_PYTHON_CSRC_LATTICE_UTILS_H_

#include "kaldifst/python/csrc/kaldifst.h"

namespace kaldifst {

void PybindLatticeUtils(py::module *m);

}  // namespace kaldifst

#endif  // KALDIFST_PYTHON_CSRC_LATTICE_UTILS_H_
